# Copyright (C) 2011 by Marek Aaron Sapota
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

try: # Python 3.x
    from configparser import SafeConfigParser
except: # Python 2.x
    from ConfigParser import SafeConfigParser
import os, os.path

class Reader():
    '''Provided variables:
    - inbox - path to inbox
    - inbox_format - format of inbox

    - ham - path to ham mailbox
    - ham_format - format of ham mailbox

    - spam - path to spam mailbox
    - spam_format - format of spam mailbox

    - notham - path to mailbox for marking as spam
    - notham_format - format of notham mailbox

    - notspam - path to mailbox for marking as ham
    - notspam_format - format of notspam mailbox

    - cleanup - command that will be run before classifying a message
    - classify - command that classifies message as spam or ham
    - learn_spam - command that learns that a message is spam
    - learn_ham - command that learns that a message is ham

    - header - header flag that is set by classify command
    - spam_header - if above header contains this value after running classify,
      message is marked as spam
    '''

    def __init__(self, path):
        def from_home(path):
            return os.path.join(os.environ['HOME'], path)

        conf = SafeConfigParser()
        conf.read(path)

        try:
            self.inbox = conf.get('Paths', 'inbox')
        except:
            self.inbox = os.path.join('/var/spool/mail', os.environ['USER'])

        try:
            self.ham = conf.get('Paths', 'ham')
        except:
            self.ham = from_home('.MailChariot/ham')

        try:
            self.spam = conf.get('Paths', 'spam')
        except:
            self.spam = from_home('.MailChariot/spam')

        try:
            self.notham = conf.get('Paths', 'notham')
        except:
            self.notham = from_home('.MailChariot/notham')

        try:
            self.notspam = conf.get('Paths', 'notspam')
        except:
            self.notspam = from_home('.MailChariot/notspam')

        try:
            self.queue = conf.get('Paths', 'queue')
        except:
            self.queue = from_home('.MailChariot/chariot-queue')

        try:
            self.inbox_format = conf.get('Format', 'inbox')
        except:
            self.inbox_format = 'mbox'

        try:
            self.ham_format = conf.get('Format', 'ham')
        except:
            self.ham_format = 'Maildir'

        try:
            self.spam_format = conf.get('Format', 'spam')
        except:
            self.spam_format = 'Maildir'

        try:
            self.notham_format = conf.get('Format', 'notham')
        except:
            self.notham_format = 'Maildir'

        try:
            self.notspam_format = conf.get('Format', 'notspam')
        except:
            self.notspam_format = 'Maildir'

        try:
            self.cleanup = conf.get('Commands', 'cleanup')
        except:
            self.cleanup = 'spamassassin -d'

        try:
            self.classify = conf.get('Commands', 'classify')
        except:
            self.classify = 'spamassassin -L'

        try:
            self.learn_spam = conf.get('Commands', 'learn_spam')
        except:
            self.learn_spam = 'sa-learn --spam'

        try:
            self.learn_ham = conf.get('Commands', 'learn_ham')
        except:
            self.learn_ham = 'sa-learn --ham'

        try:
            self.header = conf.get('Classification', 'header')
        except:
            self.header = 'X-Spam-Flag'

        try:
            self.spam_header = conf.get('Classification', 'spam')
        except:
            self.spam_header = 'YES'
