# Copyright (C) 2011 by Marek Aaron Sapota
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import Notifier
import time, mailbox, email.Errors, traceback

class Monitor():
    '''Reads emails from mailbox at given path and moves them to a queue.'''
    def __init__(self, path, queue, task):
       self.path = path
       self.queue = queue
       self.task = task
       # Read at most this many emails from inbox before releasing the lock.
       self.burst = 25

    def open(self):
        raise 'Use a specific subclass.'

    def get_messages(self):
        box = self.open()
        try:
            box.lock()
        except mailbox.ExternalClashError:
            box.close()
            return # will try again sometime later
        limit = self.burst
        for key in box.iterkeys():
            try:
                if limit < 1:
                    break
                limit -= 1
                message = box.get_string(key)
                self.queue.push(self.task, message)
                box.remove(key)
            except email.Errors.MessageError:
                print('{0} monitor - Malformed message, skipping.\n{1}'.
                        format(self.task, traceback.format_exc()))
            except:
                print('{0} monitor - Unknown error, will retry.\n{1}'.
                        format(self.task, traceback.format_exc()))
        box.flush()
        box.unlock()
        box.close()

    def __call__(self):
        while Notifier.sleep():
            self.get_messages()

class mbox(Monitor):
    def open(self):
        return mailbox.mbox(self.path)

class Maildir(Monitor):
    def open(self):
        return mailbox.Maildir(self.path)
