# Copyright (C) 2011 by Marek Aaron Sapota
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from __future__ import print_function

import OutboxWriter
from SpamAssassin import SpamAssassin
import Notifier

class RequestHandler():
    def __init__(self, conf, queue):
        def get_writer(path, format):
            if format == 'mbox':
                return OutboxWriter.mbox(path)
            elif format == 'Maildir':
                return OutboxWriter.Maildir(path)
            else:
                raise 'Unsupported mailbox format {0}.'.format(format)

        self.ham = get_writer(conf.ham, conf.ham_format)
        self.spam = get_writer(conf.spam, conf.spam_format)
        self.queue = queue
        self.header = conf.header
        self.spam_header = conf.spam_header

    def handle_request(self, task, data):
        if task == 'inbox':
            print('Classifying a message from inbox... ', end = '')
            message = SpamAssassin.classify(data)
            if (self.header in message and
                    message[self.header].find(self.spam_header) != -1):
                print('spam')
                self.spam.put_message(message)
            else:
                print('ham')
                self.ham.put_message(message)
        elif task == 'mark_ham':
            print('Marking a message as ham.')
            message = SpamAssassin.mark_ham(data)
            self.ham.put_message(message)
        elif task =='mark_spam':
            print('Marking a message as spam.')
            SpamAssassin.mark_spam(data)

    def __call__(self):
        while Notifier.sleep(0):
            request = self.queue.fetch()
            if request is not None:
                (key, task, data, ) = request
                try:
                    self.handle_request(task, data)
                    self.queue.pop(key)
                except Exception as e:
                    self.queue.release(key)
                    print('Problem with handling a task, will try again:')
                    print(e)
                    Notifier.sleep()
            else:
                Notifier.sleep()
