# Copyright (C) 2011 by Marek Aaron Sapota
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from subprocess import Popen, PIPE
import shlex
import os
import email

# That runs SpamAssassin only by default, can be configured to use anything
# else.
class SpamAssassin():
    @staticmethod
    def configure(conf):
        SpamAssassin.cleanup_cmd = conf.cleanup
        SpamAssassin.classify_cmd = conf.classify
        SpamAssassin.learn_spam_cmd = conf.learn_spam
        SpamAssassin.learn_ham_cmd = conf.learn_ham

    @staticmethod
    def __run(cmd, data):
        def preexec(): # Don't forward signals.
            os.setpgrp()

        with open(os.devnull, 'w') as devnull:
            proc = Popen(shlex.split(cmd), stdin = PIPE, stdout = PIPE,
                    stderr = devnull, preexec_fn = preexec)
            response = proc.communicate(input = data)[0]
            if proc.wait() != 0:
                raise 'Subprocess returned a non zero exit code.'
        return response

    @staticmethod
    def classify(message):
        message = SpamAssassin.__run(SpamAssassin.cleanup_cmd, message)
        message = SpamAssassin.__run(SpamAssassin.classify_cmd, message)
        return email.message_from_string(message)

    @staticmethod
    def mark_spam(message):
        message = SpamAssassin.__run(SpamAssassin.cleanup_cmd, message)
        SpamAssassin.__run(SpamAssassin.learn_spam_cmd, message)
        return email.message_from_string(message)

    @staticmethod
    def mark_ham(message):
        message = SpamAssassin.__run(SpamAssassin.cleanup_cmd, message)
        SpamAssassin.__run(SpamAssassin.learn_ham_cmd, message)
        return email.message_from_string(message)
